import time
import asyncio
from asyncio.windows_events import ProactorEventLoop as loopObject

class SelfPurgingObject(object):
    """
    The object will be auto purged from memory after a set amount of time, make sure the object has a __del__ function that handles saving if that is needed
    Requires the main function to use async
    """
    def __init__(self, event_loop: loopObject, stored_object: object, time_till_purge_in_seconds: int, reset_timer_if_accessed: bool = True):
        self.event_loop = event_loop
        self.purge_time_delay = time_till_purge_in_seconds
        self.time_to_be_deleted = event_loop.time() + self.purge_time_delay
        self.stored_object = stored_object
        self.reset_time_if_accessed = reset_timer_if_accessed
        self.current_event_loop_call = None

        self.schedule_deletion_for_purge_time()


    def reset_timer(self):
        if self.reset_time_if_accessed:
            self.time_to_be_deleted = self.event_loop.time() + self.purge_time_delay
            self.schedule_deletion_for_purge_time()
        else:
            pass # Don't reset timer duh

    def __delete_object(self):
        del self.stored_object
        del self # Make sure we are deleted

    def delete_if_time_finished_passing(self):
        if self.event_loop.time() >= self.time_to_be_deleted - 5: # 5 seconds just in case it calls it like a millisecond beforehand
            self.__delete_object()
        else:
            print("Not ready to be deleted")

    def schedule_deletion_for_purge_time(self):
        if self.current_event_loop_call is not None:
            try:
                self.current_event_loop_call.cancel()
            except():
                pass

        self.current_event_loop_call = self.event_loop.call_at(self.time_to_be_deleted, self.delete_if_time_finished_passing)